﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using BMS.Web.Models;
using BMS.Facade;
using Microsoft.Web.Mvc;
using InfoWorld.HL7.ITS;
using BMS.Web.App_GlobalResource;
using BMS.Utils;
using BMS.Facade.Data;
using System.Text;

namespace BMS.Web.Controllers
{
    /// <summary>
    /// Facility Diversion Controller class.
    /// </summary>
    [ValidateInput(false)]
    public class FacilityDiversionController : BaseController
    {
        /// <summary>
        /// Default action for Facility Diversion Controller.
        /// </summary>
        /// <returns>Returns Index view result.</returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult Index()
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                FacilityDiversionViewModel mainViewModel = new FacilityDiversionViewModel();
                mainViewModel.CurrentButton = Strings.CurrentDiversions;
                FillViewModel(mainViewModel);
                return View(mainViewModel);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Post action for Add or Edit View.
        /// </summary>
        /// <param name="mainViewModel">The main view model.</param>
        /// <returns>Returns AddConfirmation or EditConfirmation view result.</returns>
        [HttpPost]
        [UpdatePermissionAuthFilterAttribute]
        public ActionResult Index(FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                if (!string.IsNullOrEmpty(mainViewModel.ButtonAdd))                
                    return this.RedirectToAction<FacilityDiversionController>(act => act.Add(EncryptQueryString(new string[] { "location" }, new string[] { mainViewModel.Location }, loggedUser.Salt)));
                
                if (!string.IsNullOrEmpty(mainViewModel.ButtonAllDiversions))
                {
                    mainViewModel.CurrentButton = Strings.AllDiversions;                    
                    FillViewModel(mainViewModel);
                    return View(mainViewModel);
                }
                
                if (!string.IsNullOrEmpty(mainViewModel.ButtonCurrentDiversions))
                {
                    mainViewModel.CurrentButton = Strings.CurrentDiversions;
                    FillViewModel(mainViewModel);
                    return View(mainViewModel);
                }
                
                if (!string.IsNullOrEmpty(mainViewModel.ButtonShowReport))
                {
                    StringBuilder diversionsReportUrl = new StringBuilder();
                    diversionsReportUrl.Append("currentButton=");
                    diversionsReportUrl.Append(mainViewModel.CurrentButton);
                    return Redirect("/Reporting/LocalReportViewer.aspx?" + CustomEncryption.Encrypt(diversionsReportUrl.ToString(), loggedUser.Salt) + "*fd");
                }
                return null;                                                    
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        [ReadPermissionAuthFilterAttribute]
        public ActionResult Add(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                FacilityDiversionViewModel mainViewModel = new FacilityDiversionViewModel();
                mainViewModel.Location = QueryStrings["location"];
                mainViewModel.LoggedUser = this.loggedUser;
                mainViewModel.StartDate = DateTimeNowOnCurrentFacility;
                mainViewModel.HourStartDateSelected = (mainViewModel.StartDate.Value.Hour < 10) ? "0" + mainViewModel.StartDate.Value.Hour.ToString() : mainViewModel.StartDate.Value.Hour.ToString();
                mainViewModel.MinuteStartDateSelected = (mainViewModel.StartDate.Value.Minute < 10) ? "0" + mainViewModel.StartDate.Value.Minute.ToString() : mainViewModel.StartDate.Value.Minute.ToString();
                mainViewModel.Comments = string.Empty;                
                mainViewModel.FacilityAddDisplay = string.Empty;
                mainViewModel.FacilityAlreadyExists = Strings.StyleDisplayNone;
                return View(mainViewModel);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Show Add view.
        /// </summary>
        /// <param name="location">The location.</param>
        /// <returns>Returns Add view result.</returns>
        [HttpPost]
        [ReadPermissionAuthFilterAttribute]
        public ActionResult Add(FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                mainViewModel.LocationList = FacadeManager.VocabularyInterface.GetVocabulary(Util.Vocabulary.Location);
                mainViewModel.LoggedUser = this.loggedUser;
                if (!string.IsNullOrWhiteSpace(mainViewModel.StartDateMessageError))
                {
                    ModelState.AddModelError("StartDate", mainViewModel.StartDateMessageError);                    
                    mainViewModel.FacilityAddDisplay = string.Empty;
                    mainViewModel.FacilityAlreadyExists = Strings.StyleDisplayNone;

                    return View("Add", mainViewModel);
                }

                Diversion diversion = new Diversion();
                diversion.Facility = this.loggedUser.Facility;
                if (mainViewModel.LocationList != null && mainViewModel.LocationList.Count > 0)
                    diversion.Location = mainViewModel.LocationList
                                                      .First(loc => loc.displayName == mainViewModel.Location);
                diversion.StartDate = GetDate(mainViewModel.StartDate.Value, mainViewModel.HourStartDateSelected, mainViewModel.MinuteStartDateSelected);
                diversion.EndDate = diversion.StartDate;
                diversion.Comments = mainViewModel.Comments;
                diversion.Status = Constants.Yes;
                diversion.EnteredBy = this.loggedUser.UserName;
                diversion.EnteredDate = DateTimeNowOnCurrentFacility;
                diversion.LastEditBy = string.Empty;
                diversion.Id = new II { root = this.loggedUser.Domain, extension = null };

                //save in DB
                FacadeManager.DiversionInterface.SaveDiversion(diversion, true, this.loggedUser.Facility.Id);

                //display Confirmation view                
                mainViewModel.IsAddConfirmation = true;
                return View("Confirmation", mainViewModel);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Show edit view.
        /// </summary>
        /// <param name="id">The id.</param>
        /// <returns>Returns Edit view result.</returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult Edit(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                Diversion diversion = FacadeManager.DiversionInterface.FindDiversionById(this.loggedUser.Domain, QueryStrings["id"], this.loggedUser.Facility.Id);

                FacilityDiversionViewModel mainViewModel = new FacilityDiversionViewModel();
                mainViewModel.Id = QueryStrings["id"];
                mainViewModel.LoggedUser = this.loggedUser;

                mainViewModel.LocationList = FacadeManager.VocabularyInterface.GetVocabulary(Util.Vocabulary.Location).OrderByDescending(loc => loc.displayName).ToList();
                if (mainViewModel.LocationList != null && mainViewModel.LocationList.Count > 0)
                    mainViewModel.DiversionLocation = mainViewModel.LocationList.First(loc => loc.displayName == diversion.Location.displayName);
                mainViewModel.Location = diversion.Location.displayName;
                mainViewModel.StartDate = diversion.StartDate;
                mainViewModel.HourStartDateSelected = (diversion.StartDate.Hour < 10) ? "0" + diversion.StartDate.Hour.ToString() : diversion.StartDate.Hour.ToString();
                mainViewModel.MinuteStartDateSelected = (diversion.StartDate.Minute < 10) ? "0" + diversion.StartDate.Minute.ToString() : diversion.StartDate.Minute.ToString();
                mainViewModel.EndDate = diversion.EndDate;
                mainViewModel.HourEndDateSelected = (diversion.EndDate.Hour < 10) ? "0" + diversion.EndDate.Hour.ToString() : diversion.EndDate.Hour.ToString();
                mainViewModel.MinuteEndDateSelected = (diversion.EndDate.Minute < 10) ? "0" + diversion.EndDate.Minute.ToString() : diversion.EndDate.Minute.ToString();
                mainViewModel.Comments = diversion.Comments;                
                mainViewModel.FlagsList = FacadeManager.VocabularyInterface.GetVocabulary(Facade.Data.Util.Vocabulary.StrictDecision);
                mainViewModel.Status = mainViewModel.FlagsList.Find(cd => cd.code == diversion.Status);

                mainViewModel.ExistingFacilityRoot = diversion.Facility.Id.root;
                mainViewModel.ExistingFacilityExtension = diversion.Facility.Id.extension;
                mainViewModel.ExistingFacilityName = diversion.Facility.Name;

                return View(mainViewModel);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        [HttpPost]
        [UpdatePermissionAuthFilter]
        public ActionResult Edit(FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                if (!string.IsNullOrWhiteSpace(mainViewModel.StartDateMessageError))
                    AddEditModelError("StartDate", mainViewModel.StartDateMessageError, mainViewModel);
                if (!string.IsNullOrWhiteSpace(mainViewModel.EndDateMessageError))
                    AddEditModelError("EndDate", mainViewModel.EndDateMessageError, mainViewModel);
                if (!ModelState.IsValid)
                    return View("Edit", mainViewModel);
                DateTime startDate = GetDate(mainViewModel.StartDate.Value, mainViewModel.HourStartDateSelected, mainViewModel.MinuteStartDateSelected);
                DateTime endDate = GetDate(mainViewModel.EndDate.Value, mainViewModel.HourEndDateSelected, mainViewModel.MinuteEndDateSelected);
                //check to see if end date is lower than start date
                if (endDate < startDate)
                {
                    AddEditModelError("EndDate", Strings.EndDateTimeMustBeGreaterThanStartDate, mainViewModel);
                    return View("Edit", mainViewModel);
                }

                Diversion diversion = new Diversion();

                diversion.Facility = new Place { Id = new II { root = mainViewModel.ExistingFacilityRoot, extension = mainViewModel.ExistingFacilityExtension }, Name = mainViewModel.ExistingFacilityName };
                mainViewModel.LocationList = FacadeManager.VocabularyInterface.GetVocabulary(Util.Vocabulary.Location);
                if (mainViewModel.LocationList != null && mainViewModel.LocationList.Count > 0)
                    diversion.Location = mainViewModel.LocationList
                                                        .First(loc => loc.displayName == mainViewModel.DiversionLocation.displayName);
                diversion.StartDate = startDate;
                diversion.EndDate = endDate;
                diversion.Comments = mainViewModel.Comments;
                diversion.Status = mainViewModel.Status.code;
                diversion.LastEditBy = this.loggedUser.UserName;
                diversion.Id = new II { extension = mainViewModel.Id, root = this.loggedUser.Domain };

                //save in DB
                FacadeManager.DiversionInterface.SaveDiversion(diversion, false, this.loggedUser.Facility.Id);

                //display Confirmation view    
                mainViewModel.LoggedUser = loggedUser;
                mainViewModel.IsAddConfirmation = false;
                mainViewModel.Location = mainViewModel.DiversionLocation.displayName;
                return View("Confirmation", mainViewModel);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }        

        #region Private Methods

        private void FillViewModel(FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                mainViewModel.LoggedUser = this.loggedUser;
                mainViewModel.LocationList = FacadeManager.VocabularyInterface.GetVocabulary(Util.Vocabulary.Location).OrderByDescending(loc => loc.displayName).ToList();
                mainViewModel.DiversionList = FacadeManager.DiversionInterface.GetDiversions(this.loggedUser.Facility.Id).Select(d => DiversionViewModel.GetViewModel(d, DateTimeNowOnCurrentFacility)).ToList();
                if (mainViewModel.CurrentButton.Equals(Strings.AllDiversions))
                    mainViewModel.DiversionList = mainViewModel.DiversionList.OrderByDescending(fdvm => fdvm.Status).ThenByDescending(fdvm => fdvm.StartDateTime).ToList();
                else
                    mainViewModel.DiversionList = mainViewModel.DiversionList.Where(fdvm => fdvm.Status == Constants.Yes && fdvm.StartDateTime != null).OrderBy(fdvm => fdvm.StartDateTime).ToList();
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Checks if a diversion is unique.
        /// </summary>
        /// <param name="mainViewModel">Facility Diversion View Model.</param>
        /// <returns>True - diversion unique/ False - not unique</returns>
        private bool CheckUnique(FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                mainViewModel.LoggedUser = this.loggedUser;
                mainViewModel.Comments = mainViewModel.Comments ?? string.Empty;
                Diversion diversion = FacadeManager.DiversionInterface.GetDiversions(this.loggedUser.Facility.Id)
                                                                      .Where(d => d.Facility.Id.extension.Equals(this.loggedUser.Facility.Id.extension, StringComparison.InvariantCultureIgnoreCase) && d.Location.displayName == mainViewModel.Location && d.Comments.ToLower() == mainViewModel.Comments.ToLower())
                                                                      .FirstOrDefault();

                if (diversion != null)
                {
                    mainViewModel.FacilityAddDisplay = Strings.StyleDisplayNone;
                    mainViewModel.FacilityAlreadyExists = string.Empty;
                    return true;
                }
                else
                {
                    mainViewModel.FacilityAddDisplay = string.Empty;
                    mainViewModel.FacilityAlreadyExists = Strings.StyleDisplayNone;
                    return false;
                }

            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Gets the date.
        /// </summary>
        /// <param name="dateTime">The date time.</param>
        /// <param name="hour">The hour.</param>
        /// <param name="minutes">The minutes.</param>
        /// <returns></returns>
        private DateTime GetDate(DateTime dateTime, string hour, string minutes)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                return new DateTime(dateTime.Year, dateTime.Month,
                                                   dateTime.Day,
                                                   Convert.ToInt32(hour),
                                                   Convert.ToInt32(minutes), 0);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        private void AddEditModelError(string propertyName, string errorMessage, FacilityDiversionViewModel mainViewModel)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                ModelState.AddModelError(propertyName, errorMessage);
                mainViewModel.FlagsList = FacadeManager.VocabularyInterface.GetVocabulary(Facade.Data.Util.Vocabulary.StrictDecision);
                mainViewModel.LocationList = FacadeManager.VocabularyInterface.GetVocabulary(Util.Vocabulary.Location);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }
        #endregion
    }
}