﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using BMS.Facade.Data;
using BMS.Web.Models;
using InfoWorld.HL7.ITS;
using BMS.Web.App_GlobalResource;
using Microsoft.Web.Mvc;
using BMS.Facade;
using BMS.Web.Controllers.Shared;
namespace BMS.Web.Controllers
{
    /// <summary>
    /// Ward Configuration Controller class.
    /// </summary>
    [ValidateInput(false)]
    public class WardConfigurationController : BaseController
    {
        #region Public Methods

        /// <summary>
        ///  Default action for the WardConfiguration controller.
        /// </summary>
        /// <returns></returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult Index(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                WardConfigurationViewModel model = new WardConfigurationViewModel();

                FillModel(model);

                if (string.IsNullOrEmpty(QueryStrings["wardId"]))
                    SetDefaultValueForDropDownList(model);
                else
                {
                    model.RootId = loggedUser.Domain;
                    model.ExtensionId = QueryStrings["wardId"];

                    WardViewModel ward = model.ActiveWards.Where(a => a.Model.Id.root.Equals(loggedUser.Domain, StringComparison.InvariantCultureIgnoreCase) && a.Model.Id.extension.Equals(QueryStrings["wardId"], StringComparison.InvariantCultureIgnoreCase)).FirstOrDefault();
                    model.SelectedWardExtension = ward == null ? null : ward.Model.Id.extension;
                    model.TypeGroup = ward == null ? null : ward.TypeGroup;
                    model.WardGroupText = ward == null ? null : ward.DivisionName;
                }

                if (TempData.ContainsKey("ModelState"))
                {
                    ModelStateDictionary externalModelState = (ModelStateDictionary)TempData["ModelState"];
                    foreach (KeyValuePair<string, ModelState> valuePair in externalModelState)
                        ModelState.Add(valuePair.Key, valuePair.Value);
                }
                return View(model);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Post action for the WardConfiguration controller.
        /// </summary>
        /// <param name="model">The model.</param>
        /// <param name="btnSave">The BTN save.</param>
        /// <returns></returns>
        [HttpPost]
        [UpdatePermissionAuthFilterAttribute]
        public ActionResult Index(WardConfigurationViewModel model)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                if (!string.IsNullOrEmpty(model.SelectedWardExtension) && !model.SelectedWardExtension.Equals(Guid.Empty.ToString()))
                {
                    FillModel(model);
                    Ward ward = Facade.FacadeManager.EntityInterface.GetWardsByVista(this.loggedUser.VistaSite.Id).Where(a => a.Id.extension.Equals(model.SelectedWardExtension, StringComparison.InvariantCultureIgnoreCase)).FirstOrDefault();
                    if (ward != null)
                    {
                        bool isAdded = false;
                        ward.TypeGroup = model.TypeGroup.ToUpper();
                        if (ward.Division == null || ward.Division.Name != model.WardGroupText.ToUpper())
                        {
                            if (ward.Division == null)
                                isAdded = true;
                            ward.Division = new Division { Name = model.WardGroupText.ToUpper() };
                        }
                        Facade.FacadeManager.EntityInterface.UpdateWard(ward);
                        return this.RedirectToAction<WardConfigurationController>(act => act.AddEditWardConfirmation(EncryptQueryString(new string[] { "wardId", "sortByWard", "sortBySpecialty", "sortByTypeGroup", "sortByWardGroup", "isAddedMessageConfirmation" }, new string[] { ward.Id.extension, model.SortByVistaWardNameButton, model.SortByVistaSpecialtyButton, model.SortByVistaTypeGroupButton, model.SortByVistaWardGroupTextButton, isAdded.ToString() }, loggedUser.Salt)));
                    }
                    ModelState.Clear();
                    FillModel(model);
                    SetDefaultValueForDropDownList(model);
                }
                else
                {
                    ModelState.AddModelError("SelectedWardExtension", Strings.YouMustSelectAWard);
                    //Store modelstate from tempdata
                    TempData.Add("ModelState", ModelState);
                    FillModel(model);
                }
                return View(model);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Go to delete page.
        /// </summary>
        /// <param name="ien">The ien.</param>
        /// <returns></returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult Delete(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                Ward ward = FacadeManager.EntityInterface.GetWardById(new II() { root = loggedUser.Domain, extension = QueryStrings["wardId"] });
                AntiXssEncoder xss = new AntiXssEncoder();
                ward.TypeGroup = xss.Decode(ward.TypeGroup);
                ward.Division.Name = xss.Decode(ward.Division.Name);
                WardViewModel model = new WardViewModel() { Model = ward };
                model.LoggedUser = loggedUser;
                return View(model);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Delete the ward.
        /// </summary>
        /// <param name="identifier">The identifier.</param>
        /// <returns></returns>
        [HttpPost]
        [UpdatePermissionAuthFilterAttribute]
        public ActionResult DeleteWard(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                Facade.FacadeManager.EntityInterface.DeleteWard(new II() { root = loggedUser.Domain, extension = QueryStrings["wardId"] });
                return this.RedirectToAction<WardConfigurationController>(act => act.DeleteConfirmation(p));
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Display delete confirmation page.
        /// </summary>
        /// <param name="ien">The ien.</param>
        /// <param name="wardName">Name of the ward.</param>
        /// <param name="typeGroup">The type group.</param>
        /// <param name="wardGroup">The ward group.</param>
        /// <returns>DeleteConfirmation view.</returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult DeleteConfirmation(string p)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                WardViewModel model = new WardViewModel();
                model.Model = new Ward();
                model.Model.Ien = QueryStrings["ien"];
                model.Model.Name = QueryStrings["wardName"];
                model.Model.TypeGroup = QueryStrings["typeGroup"];
                model.Model.Division = new Division() { Name = QueryStrings["wardGroup"] };
                model.LoggedUser = loggedUser;
                return View(model);
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Display update (add\edit) ward confirmation page.
        /// </summary>
        /// <param name="rootId">RootId of ward.</param>
        /// <param name="extensionId">ExtensionId of ward.</param>
        /// <param name="sortByVistaWardNameButton">Sort by VistA ward name.</param>
        /// <param name="sortByVistaSpecialtyButton">Sort by VistA specialty.</param>
        /// <param name="sortByVistaTypeGroupButton">Sort by type group.</param>
        /// <param name="sortByVistaWardGroupTextButton">Sort by division.</param>
        /// <param name="isAddedMessageConfirmation">True if is added new ward; otherwise,false.</param>
        /// <returns>DeleteConfirmation view.</returns>
        [ReadPermissionAuthFilterAttribute]
        public ActionResult AddEditWardConfirmation(string p)
        {
            AddEditWardViewModel model = new AddEditWardViewModel();
            model.SortByVistaWardNameButton = QueryStrings["sortByWard"];
            model.SortByVistaSpecialtyButton = QueryStrings["sortBySpecialty"];
            model.SortByVistaTypeGroupButton = QueryStrings["sortByTypeGroup"];
            model.SortByVistaWardGroupTextButton = QueryStrings["sortByWardGroup"];
            model.IsAddedMessageConfirmation = bool.Parse(QueryStrings["isAddedMessageConfirmation"]);
            model.RootId = loggedUser.Domain;
            model.ExtensionId = QueryStrings["wardId"];
            model.LoggedUser = loggedUser;
            Ward ward = FacadeManager.EntityInterface.GetWardById(new II() { root = model.RootId, extension = model.ExtensionId });
            AntiXssEncoder xss = new AntiXssEncoder();
            ward.TypeGroup = xss.Decode(ward.TypeGroup);
            ward.Division.Name = xss.Decode(ward.Division.Name);
            model.Ward = new WardViewModel() { Model = ward };
            return View(model);
        }

        /// <summary>
        /// Sorts the wards.
        /// </summary>
        /// <param name="buttonId">The button id.</param>
        /// <returns>Wards partial view.</returns>
        [HttpPost]
        public ActionResult SortWards(string buttonId)
        {
            AntiXssEncoder xss = new AntiXssEncoder();
            WardConfigurationViewModel model = new WardConfigurationViewModel();
            model.LoggedUser = loggedUser;

            model.ActiveWards = Facade.FacadeManager.EntityInterface.GetWardsByFacility(this.loggedUser.Facility.Id).Where(a => a.IsActive == true && a.Division != null).Select(x => new WardViewModel() { Model = x }).ToList();
            model.ActiveWards.ToList().ForEach(a => { a.Model.TypeGroup = xss.Decode(a.Model.TypeGroup); a.Model.Division.Name = xss.Decode(a.Model.Division.Name); });

            switch (buttonId)
            {
                case "VistaWardName":
                    model.ActiveWards = model.ActiveWards.OrderBy(a => a.Model.Name).ToList();
                    break;
                case "VistaSpeciality":
                    model.ActiveWards = model.ActiveWards.OrderBy(a => a.Model.Specialty.displayName).ToList();
                    break;
                case "VistaTypeGroup":
                    model.ActiveWards = model.ActiveWards.OrderBy(a => a.Model.TypeGroup).ToList();
                    break;
                case "VistaWardGroupText":
                    model.ActiveWards = model.ActiveWards.OrderBy(a => a.Model.Division.Name).ToList();
                    break;
            }

            return PartialView("Wards", model);
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Fill the model.
        /// </summary>
        /// <param name="model">The model.</param>
        private void FillModel(WardConfigurationViewModel model)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                AntiXssEncoder xss = new AntiXssEncoder();
                model.Wards = Facade.FacadeManager.EntityInterface.GetWardsByVista(this.loggedUser.VistaSite.Id).Where(a => a.IsCurrentlyOOS == false && String.Compare(a.Name, "0") > 0 && String.Compare(a.Name, "ZY") < 0).Select(x => new WardViewModel() { Model = new Ward() { Address1 = x.Address1, Division = x.Division, Id = x.Id, Ien = x.Ien, IsActive = x.IsActive, Name = x.Name + " {" + Strings.IEN + ":" + x.Ien.ToString() + "}", Specialty = x.Specialty, TypeGroup = x.TypeGroup } }).ToList();
                model.ActiveWards = Facade.FacadeManager.EntityInterface.GetWardsByFacility(this.loggedUser.Facility.Id).Where(a => a.IsActive == true && a.Division != null).Select(x => new WardViewModel() { Model = x }).ToList();
                model.ActiveWards.ToList().ForEach(a => { a.Model.TypeGroup = xss.Decode(a.Model.TypeGroup); a.Model.Division.Name = xss.Decode(a.Model.Division.Name); });
                //Add Select A WARD option to the drop down list.
                model.Wards.Insert(0, new WardViewModel() { Model = new Ward() { Name = Strings.SelectAWardUpperCase, Id = new II() { root = Strings.SelectAWardUpperCase, extension = Guid.Empty.ToString() } } });
                //Add the current facility name to the model.
                model.FacilityName = this.loggedUser.Facility.Name;
                model.LoggedUser = loggedUser;
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        /// <summary>
        /// Sets the default value for drop down list.
        /// </summary>
        /// <param name="model">The model.</param>
        private void SetDefaultValueForDropDownList(WardConfigurationViewModel model)
        {
            DateTime entryInLogMethodTime = DateTime.UtcNow;
            if (InfoWorld.Tracing.IWTrace.IsEntryEnabled)
            {
                InfoWorld.Tracing.IWTrace.Entry(System.Reflection.MethodBase.GetCurrentMethod(), entryInLogMethodTime);
            }
            try
            {
                model.SelectedWardExtension = Guid.Empty.ToString();
                model.TypeGroup = string.Empty;
                model.WardGroupText = string.Empty;
            }
            finally
            {
                if (InfoWorld.Tracing.IWTrace.IsExitEnabled)
                {
                    InfoWorld.Tracing.IWTrace.Exit(System.Reflection.MethodBase.GetCurrentMethod(), DateTime.UtcNow, entryInLogMethodTime);
                }
            }
        }

        #endregion
    }
}
